
/* A Bison parser, made by GNU Bison 2.4.1.  */

/* Skeleton implementation for Bison's Yacc-like parsers in C
   
      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
   Free Software Foundation, Inc.
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.
   
   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* C LALR(1) parser skeleton written by Richard Stallman, by
   simplifying the original so-called "semantic" parser.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

/* Identify Bison output.  */
#define YYBISON 1

/* Bison version.  */
#define YYBISON_VERSION "2.4.1"

/* Skeleton name.  */
#define YYSKELETON_NAME "yacc.c"

/* Pure parsers.  */
#define YYPURE 0

/* Push parsers.  */
#define YYPUSH 0

/* Pull parsers.  */
#define YYPULL 1

/* Using locations.  */
#define YYLSP_NEEDED 0



/* Copy the first part of user declarations.  */

/* Line 189 of yacc.c  */
#line 1 "compiler.y"

/*

    Ann Hell Ex Machina - Music Software
    Copyright (C) 2003/2008 Angel Ortega <angel@triptico.com>

    compiler.y - Scripting language YACC parser

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    http://www.triptico.com

*/

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "ahxm.h"


/** data **/

int yylex(void);
void yyerror(const char *s);

/* injecting code functions (defined in compiler.l) */
int push_code(const char *code);
int push_code_from_file(const char *file);

/* current track */
int track;

/* current time */
double cur_time = 0.0;

/* end time (longest time seen) */
double end_time = 0.0;

/* note globals */
static double length;
static int octave;
static int transpose;
static double staccato;
static sample_t volume_from;
static sample_t volume_to;
static double gliss;

/* parser debugging */

#define DEBUGF if(verbose >= 2)printf

/* blocks */

struct block {
    const char *name;
    int n_sblocks;
    char **sblocks;
};

static struct block *blocks = NULL;
static int n_blocks = 0;

/* group info */
struct group {
    double start;
    double gliss;
    double max;
};

static struct group *groups = NULL;
static int n_groups = 0;
static int groups_size = 0;

/* mark info */
struct mark {
    const char *name;
    double time;
};

static struct mark *marks = NULL;
static int n_marks = 0;

/* arpeggiator */
struct arp {
    double delay;
    int transpose;
    sample_t volume;
    int track_off;
};

static struct arp *arps = NULL;
static int n_arps = 0;
static int arps_size = 0;

static double arp_delay;
static int arp_transpose;
static sample_t arp_volume;
static int arp_track_off;

int compiler_error = 0;

extern int yyline;

/* alterations for each note */
int alterations[12];

/* random seeds */
unsigned int block_seed = 0;
unsigned int volume_seed = 0;

/* solo track (-1, no solo) */
int solo_track = -1;


/** code **/

unsigned int ah_rnd(unsigned int *seed)
/* special randomizer */
{
    *seed = (*seed * 58321) + 11113;

    return *seed >> 16;
}


void c_err(const char *e1, const char *e2, const char *e3)
/* reports an error from the compiler */
{
    printf("ahxm:");

    if (e1 != NULL)
        printf(" %s", e1);
    if (e2 != NULL)
        printf(" %s", e2);
    if (e3 != NULL)
        printf(" %s", e3);

    printf(" in line %d\n", yyline);

    compiler_error++;
}


static void forward(double step)
/* moves forward current time */
{
    /* add step */
    cur_time += step;

    /* quantizations could be done here */
}


static sample_t volume(void)
{
    sample_t f;

    if (volume_from == volume_to)
        f = volume_from;
    else {
        unsigned int r = ah_rnd(&volume_seed) % 1000;

        f = volume_from + (volume_to - volume_from) *
            (((sample_t) r) / 1000.0);
    }

    return f;
}


/* blocks */

static int find_block(const char *name)
/* finds a block */
{
    int n;

    for (n = 0; n < n_blocks; n++) {
        if (strcmp(name, blocks[n].name) == 0)
            return n;
    }

    return -1;
}


static int set_block(const char *name, char *block)
/* defines a block */
{
    int n;
    struct block *b;
    char *start;
    char *stop;

    /* if block already exists, free it */
    if ((n = find_block(name)) >= 0) {
        b = &blocks[n];

        /* free all subblocks */
        for (n = 0; n < b->n_sblocks; n++)
            free(b->sblocks[n]);

        /* free the subblock array */
        free(b->sblocks);
    }
    else {
        GROW(blocks, n_blocks, struct block);
        b = &blocks[n_blocks++];

        b->name = strdup(name);
    }

    /* reset */
    b->n_sblocks = 0;
    b->sblocks = NULL;

    /* now split in subblocks (delimited by : ) */
    start = block;

    while ((stop = strchr(start, ':')) != NULL) {
        /* break there */
        *stop = '\0';

        /* add the subblock */
        GROW(b->sblocks, b->n_sblocks, char *);
        b->sblocks[b->n_sblocks++] = strdup(start);

        start = stop + 1;
    }

    /* no more separators? store the rest */
    GROW(b->sblocks, b->n_sblocks, char *);
    b->sblocks[b->n_sblocks++] = strdup(start);

    /* the original block is no longer needed */
    free(block);

    return 1;
}


static void insert_block(const char *name)
/* inserts a block */
{
    int n;

    if ((n = find_block(name)) >= 0) {
        struct block *b;

        b = &blocks[n];

        /* get one of them, randomly */
        n = ah_rnd(&block_seed) % b->n_sblocks;

        push_code(strdup(b->sblocks[n]));
    }
    else
        c_err("block-not-found", name, NULL);
}


static int insert_file(const char *filename)
{
    if (!push_code_from_file(filename)) {
        c_err("script-not-found", filename, NULL);
        return 1;
    }

    return 0;
}


/* groups */

static int push_group(void)
/* starts a new group of notes */
{
    struct group *g;

    if (n_groups == groups_size) {
        GROW(groups, groups_size, struct group);
        groups_size++;
    }

    g = &groups[n_groups++];

    g->start = cur_time;
    g->gliss = 0.0;
    g->max = 0.0;

    return 1;
}


static int next_group_part(void)
/* part delimiter */
{
    struct group *g;

    if (n_groups == 0) {
        c_err("missing-start-of-group", NULL, NULL);
        return 0;
    }

    g = &groups[n_groups - 1];

    /* store maximum frame */
    if (g->max < cur_time)
        g->max = cur_time;

    /* add glissando delay */
    g->gliss += gliss;

    /* rewind */
    cur_time = g->start + g->gliss;

    return 1;
}


static int pop_group(void)
/* finishes a group, moving the frame to the end of the longer part */
{
    if (n_groups == 0) {
        c_err("missing-start-of-group", NULL, NULL);
        return 0;
    }

    n_groups--;

    /* if other parts of group were longer than the last one,
       move pointer there */
    if (groups[n_groups].max > cur_time)
        cur_time = groups[n_groups].max;

    return 1;
}


/* marks */

static int seek_mark(const char *name)
/* seeks a mark by name; returns its offset or -1 */
{
    int n;

    for (n = 0; n < n_marks; n++)
        if (strcmp(marks[n].name, name) == 0)
            return n;

    return -1;
}


static void add_mark(const char *name)
/* creates a new mark */
{
    int n;

    if ((n = seek_mark(name)) == -1) {
        n = n_marks++;
        GROW(marks, n, struct mark);
        marks[n].name = strdup(name);
    }

    marks[n].time = cur_time;
}


static void find_mark(const char *name, int set)
/* finds a mark by name, optionally moving time cursor there */
{
    int n;

    if ((n = seek_mark(name)) != -1) {
        if (set) {
            if (cur_time > marks[n].time) {
                /* if mark is not END, fail */
                if (strcmp(name, "END") != 0)
                    c_err("mark-overpassed", name, NULL);
            }
            else
                cur_time = marks[n].time;
        }
        else
        if (cur_time != marks[n].time)
            c_err("mark-mismatch", name, NULL);

        return;
    }

    c_err("mark-not-found", name, NULL);
}


/* arpeggiator */

static void arp_default(void)
/* resets arpeggiator values to the default ones */
{
    arp_delay = 0.0;
    arp_transpose = 0;
    arp_volume = 1.0;
    arp_track_off = 0;
}


static void add_arp(void)
/* adds an arpeggiator note */
{
    struct arp *a;

    /* if the note is exactly the same, do nothing */
    if (arp_delay == 0.0 && arp_transpose == 0 &&
        arp_volume == 1.0 && arp_track_off == 0)
        return;

    if (n_arps == arps_size) {
        GROW(arps, arps_size, struct arp);
        arps_size++;
    }

    a = &arps[n_arps];

    a->delay = arp_delay;
    a->transpose = arp_transpose;
    a->volume = arp_volume;
    a->track_off = arp_track_off;

    n_arps++;
    arp_default();
}


static void set_alteration(const char *altstr)
/* sets the alterations from altstr */
{
    int n, steps[] = { 2, 0, 2, 0, 1, 2, 0, 2, 0, 2, 0, 1 };

    /* reset alterations */
    for (n = 0; n < 12; n++)
        alterations[n] = 0;

    /* changed according the altstr spec */
    for (n = 0; *altstr != '\0' && n < 12; altstr++) {
        switch (*altstr) {
        case '&':
            alterations[n] = -1;
            break;

        case '#':
            alterations[n] = 1;
            break;
        }

        /* move to next natural note */
        n += steps[n];
    }
}


/* song events */

static struct song_ev *add_song_ev(song_ev_type type)
{
    struct song_ev *r;

    r = add_event(&song, &n_song_ev);

    r->type = type;
    r->time = cur_time;
    r->trk_id = track;
    r->event_id = n_song_ev - 1;

    return r;
}


static void add_note_event(int note)
/* adds a note event */
{
    int n;
    int np;
    struct song_ev *e;

    /* sum the alteration */
    if ((n = note % 12) < 0)
        n += 12;

    note += alterations[n];

    /* calculate the note */
    np = note + transpose + (octave * 12);

    /* is note out of range? */
    if (np < 0 || np > 127) {
        c_err("note-out-of-range", NULL, NULL);
        return;
    }

    e = add_song_ev(SONG_EV_NOTE);

    e->value = np;
    e->len = length * staccato;
    e->vol = volume();

    /* add arpeggiator repetitions */
    for (n = 0; n < n_arps; n++) {

        e = add_song_ev(SONG_EV_NOTE);

        e->time = cur_time + arps[n].delay;
        e->trk_id = track + arps[n].track_off;
        e->value = np + arps[n].transpose;
        e->len = length * staccato;
        e->vol = volume() * arps[n].volume;
    }
}


static void add_back_event(void)
{
    struct song_ev *e = add_song_ev(SONG_EV_BACK);

    e->len = length;
}


static void add_tempo_event(int trk_id, double tempo)
{
    struct song_ev *e = add_song_ev(SONG_EV_TEMPO);

    e->trk_id = trk_id;
    e->amount = tempo;
}


static void add_meter_event(int trk_id, int num, int den)
{
    struct song_ev *e = add_song_ev(SONG_EV_METER);

    e->trk_id = trk_id;
    e->min = num;
    e->max = den;
}

static void add_measure_event(void)
{
    struct song_ev *e = add_song_ev(SONG_EV_MEASURE);

    e->trk_id = -1;
    e->value = yyline;
}


static void add_ss_sustain_event(double sustain)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_SUSTAIN);

    e->amount = sustain;
}


static void add_ss_attack_event(double attack)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_ATTACK);

    e->amount = attack;
}


static void add_ss_vibrato_event(double depth, double freq)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_VIBRATO);

    e->depth = depth;
    e->freq = freq;
}


static void add_ss_portamento_event(double portamento)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_PORTAMENTO);

    e->amount = portamento;
}


static void add_ss_channel_event(int channel, sample_t vol)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_CHANNEL);

    e->channel = channel;
    e->vol = vol;
}


static void add_ss_wav_event(const char *wav_file, int base, int min,
                             int max, double loop_start, double loop_end,
                             int first_channel, int skip_channels)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_WAV);

    e->name = wav_file;
    e->value = base;
    e->min = min;
    e->max = max;
    e->start = loop_start;
    e->end = loop_end;
    e->channel = first_channel;
    e->skip_channels = skip_channels;
}


static void add_ss_pat_event(const char *pat_file)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_PAT);

    e->name = pat_file;
}


static void add_ss_sf2_event(const char *sf2_file, const char *insname)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_SF2);

    e->name = sf2_file;
    e->str2 = insname;
}


static void add_ss_eff_event(int type, int channel, double size,
                             sample_t gain, double depth, double freq,
                             double phase, sample_t initial,
                             sample_t final)
{
    struct song_ev *e = add_song_ev(type);

    e->channel = channel;
    e->len = size;
    e->vol = gain;
    e->depth = depth;
    e->freq = freq;
    e->phase = phase;
    e->initial = initial;
    e->final = final;
}


static void add_ss_pitch_stretch(int note, double len, sample_t vol)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_PITCH_STRETCH);

    e->value = note;
    e->len = len;
    e->vol = vol;
}


static void add_ss_print_wave_tempo(int note, double len)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_PRINT_WAVE_TEMPO);

    e->value = note;
    e->len = len;
}


static void add_ss_master_volume(sample_t vol)
{
    struct song_ev *e = add_song_ev(SONG_EV_SS_MASTER_VOLUME);

    e->trk_id = -1;
    e->vol = vol;
}


static void add_midi_channel_event(int channel)
{
    struct song_ev *e = add_song_ev(SONG_EV_MIDI_CHANNEL);

    e->channel = channel - 1;
}


static void add_midi_program_event(int program)
{
    struct song_ev *e = add_song_ev(SONG_EV_MIDI_PROGRAM);

    e->value = program;
}


static void add_song_info_event(const char *author, const char *name)
{
    struct song_ev *e = add_song_ev(SONG_EV_SONG_INFO);

    e->str2 = author;
    e->name = name;
}


static void init_track(void)
/* sets the default values for a new track */
{
    int n;

    /* is there an end time? test if this is longer */
    if (cur_time > end_time) {
        add_mark("END");
        end_time = cur_time;
    }

    cur_time = 0.0;
    length = 0.0;
    transpose = 0;
    staccato = 0.8;
    volume_from = volume_to = 0.75;
    octave = 5;
    gliss = 0;

    /* groups should not cross track boundaries */
    n_groups = 0;

    /* reset arpeggiator */
    n_arps = 0;
    arp_default();

    /* is there a START mark? if so, move there */
    if ((n = seek_mark("START")) != -1)
        cur_time = marks[n].time;

    /* reset alterations */
    for (n = 0; n < 12; n++)
        alterations[n] = 0;
}



/* Line 189 of yacc.c  */
#line 829 "y.tab.c"

/* Enabling traces.  */
#ifndef YYDEBUG
# define YYDEBUG 0
#endif

/* Enabling verbose error messages.  */
#ifdef YYERROR_VERBOSE
# undef YYERROR_VERBOSE
# define YYERROR_VERBOSE 1
#else
# define YYERROR_VERBOSE 0
#endif

/* Enabling the token table.  */
#ifndef YYTOKEN_TABLE
# define YYTOKEN_TABLE 0
#endif


/* Tokens.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
   /* Put the tokens into the symbol table, so that GDB and other debuggers
      know about them.  */
   enum yytokentype {
     P_INTEGER = 258,
     S_INTEGER = 259,
     P_REAL = 260,
     S_REAL = 261,
     NOTE_P = 262,
     NOTE_T3 = 263,
     NOTE_T5 = 264,
     NEW_MARK = 265,
     GOTO_MARK = 266,
     ASSERT_MARK = 267,
     BLOCK = 268,
     BLK_ASSIGN = 269,
     BLK_INSERT = 270,
     FILE_INSERT = 271,
     ALTSTR = 272,
     LIBPATH_ADD = 273,
     XC_STR = 274,
     XC_ABSNOTE = 275,
     XC_MSECS = 276,
     SS_SEP = 277,
     SS_WAV = 278,
     SS_LOOP_WAV = 279,
     SS_PAT = 280,
     SS_SF2 = 281,
     SS_SUSTAIN = 282,
     SS_ATTACK = 283,
     SS_VIBRATO = 284,
     SS_PORTAMENTO = 285,
     SS_CHANNEL = 286,
     SS_VOL = 287,
     SS_MASTER_VOL = 288,
     SS_EFF_DELAY = 289,
     SS_EFF_ECHO = 290,
     SS_EFF_COMB = 291,
     SS_EFF_ALLPASS = 292,
     SS_EFF_FLANGER = 293,
     SS_EFF_WOBBLE = 294,
     SS_EFF_SQWOBBLE = 295,
     SS_EFF_HFWOBBLE = 296,
     SS_EFF_FADER = 297,
     SS_EFF_REVERB = 298,
     SS_EFF_FOLDBACK = 299,
     SS_EFF_ATAN = 300,
     SS_EFF_DISTORT = 301,
     SS_EFF_OVERDRIVE = 302,
     SS_EFF_OFF = 303,
     SS_PITCH_STRETCH = 304,
     SS_TIME_STRETCH = 305,
     SS_PRINT_WAVE_TEMPO = 306,
     SONG_INFO = 307,
     MIDI_CHANNEL = 308,
     MIDI_PROGRAM = 309,
     MIDI_GENERIC = 310
   };
#endif
/* Tokens.  */
#define P_INTEGER 258
#define S_INTEGER 259
#define P_REAL 260
#define S_REAL 261
#define NOTE_P 262
#define NOTE_T3 263
#define NOTE_T5 264
#define NEW_MARK 265
#define GOTO_MARK 266
#define ASSERT_MARK 267
#define BLOCK 268
#define BLK_ASSIGN 269
#define BLK_INSERT 270
#define FILE_INSERT 271
#define ALTSTR 272
#define LIBPATH_ADD 273
#define XC_STR 274
#define XC_ABSNOTE 275
#define XC_MSECS 276
#define SS_SEP 277
#define SS_WAV 278
#define SS_LOOP_WAV 279
#define SS_PAT 280
#define SS_SF2 281
#define SS_SUSTAIN 282
#define SS_ATTACK 283
#define SS_VIBRATO 284
#define SS_PORTAMENTO 285
#define SS_CHANNEL 286
#define SS_VOL 287
#define SS_MASTER_VOL 288
#define SS_EFF_DELAY 289
#define SS_EFF_ECHO 290
#define SS_EFF_COMB 291
#define SS_EFF_ALLPASS 292
#define SS_EFF_FLANGER 293
#define SS_EFF_WOBBLE 294
#define SS_EFF_SQWOBBLE 295
#define SS_EFF_HFWOBBLE 296
#define SS_EFF_FADER 297
#define SS_EFF_REVERB 298
#define SS_EFF_FOLDBACK 299
#define SS_EFF_ATAN 300
#define SS_EFF_DISTORT 301
#define SS_EFF_OVERDRIVE 302
#define SS_EFF_OFF 303
#define SS_PITCH_STRETCH 304
#define SS_TIME_STRETCH 305
#define SS_PRINT_WAVE_TEMPO 306
#define SONG_INFO 307
#define MIDI_CHANNEL 308
#define MIDI_PROGRAM 309
#define MIDI_GENERIC 310




#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
typedef union YYSTYPE
{

/* Line 214 of yacc.c  */
#line 756 "compiler.y"

	int i;
	double d;
	char * p;



/* Line 214 of yacc.c  */
#line 983 "y.tab.c"
} YYSTYPE;
# define YYSTYPE_IS_TRIVIAL 1
# define yystype YYSTYPE /* obsolescent; will be withdrawn */
# define YYSTYPE_IS_DECLARED 1
#endif


/* Copy the second part of user declarations.  */


/* Line 264 of yacc.c  */
#line 995 "y.tab.c"

#ifdef short
# undef short
#endif

#ifdef YYTYPE_UINT8
typedef YYTYPE_UINT8 yytype_uint8;
#else
typedef unsigned char yytype_uint8;
#endif

#ifdef YYTYPE_INT8
typedef YYTYPE_INT8 yytype_int8;
#elif (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
typedef signed char yytype_int8;
#else
typedef short int yytype_int8;
#endif

#ifdef YYTYPE_UINT16
typedef YYTYPE_UINT16 yytype_uint16;
#else
typedef unsigned short int yytype_uint16;
#endif

#ifdef YYTYPE_INT16
typedef YYTYPE_INT16 yytype_int16;
#else
typedef short int yytype_int16;
#endif

#ifndef YYSIZE_T
# ifdef __SIZE_TYPE__
#  define YYSIZE_T __SIZE_TYPE__
# elif defined size_t
#  define YYSIZE_T size_t
# elif ! defined YYSIZE_T && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# else
#  define YYSIZE_T unsigned int
# endif
#endif

#define YYSIZE_MAXIMUM ((YYSIZE_T) -1)

#ifndef YY_
# if YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
#   define YY_(msgid) dgettext ("bison-runtime", msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(msgid) msgid
# endif
#endif

/* Suppress unused-variable warnings by "using" E.  */
#if ! defined lint || defined __GNUC__
# define YYUSE(e) ((void) (e))
#else
# define YYUSE(e) /* empty */
#endif

/* Identity function, used to suppress warnings about constant conditions.  */
#ifndef lint
# define YYID(n) (n)
#else
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static int
YYID (int yyi)
#else
static int
YYID (yyi)
    int yyi;
#endif
{
  return yyi;
}
#endif

#if ! defined yyoverflow || YYERROR_VERBOSE

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# ifdef YYSTACK_USE_ALLOCA
#  if YYSTACK_USE_ALLOCA
#   ifdef __GNUC__
#    define YYSTACK_ALLOC __builtin_alloca
#   elif defined __BUILTIN_VA_ARG_INCR
#    include <alloca.h> /* INFRINGES ON USER NAME SPACE */
#   elif defined _AIX
#    define YYSTACK_ALLOC __alloca
#   elif defined _MSC_VER
#    include <malloc.h> /* INFRINGES ON USER NAME SPACE */
#    define alloca _alloca
#   else
#    define YYSTACK_ALLOC alloca
#    if ! defined _ALLOCA_H && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#     ifndef _STDLIB_H
#      define _STDLIB_H 1
#     endif
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's `empty if-body' warning.  */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (YYID (0))
#  ifndef YYSTACK_ALLOC_MAXIMUM
    /* The OS might guarantee only one guard page at the bottom of the stack,
       and a page size can be as small as 4096 bytes.  So we cannot safely
       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
       to allow for a few compiler-allocated temporary stack slots.  */
#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2006 */
#  endif
# else
#  define YYSTACK_ALLOC YYMALLOC
#  define YYSTACK_FREE YYFREE
#  ifndef YYSTACK_ALLOC_MAXIMUM
#   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
#  endif
#  if (defined __cplusplus && ! defined _STDLIB_H \
       && ! ((defined YYMALLOC || defined malloc) \
	     && (defined YYFREE || defined free)))
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   ifndef _STDLIB_H
#    define _STDLIB_H 1
#   endif
#  endif
#  ifndef YYMALLOC
#   define YYMALLOC malloc
#   if ! defined malloc && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
#  ifndef YYFREE
#   define YYFREE free
#   if ! defined free && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
void free (void *); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
# endif
#endif /* ! defined yyoverflow || YYERROR_VERBOSE */


#if (! defined yyoverflow \
     && (! defined __cplusplus \
	 || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  yytype_int16 yyss_alloc;
  YYSTYPE yyvs_alloc;
};

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# define YYSTACK_BYTES(N) \
     ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
      + YYSTACK_GAP_MAXIMUM)

/* Copy COUNT objects from FROM to TO.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if defined __GNUC__ && 1 < __GNUC__
#   define YYCOPY(To, From, Count) \
      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
#  else
#   define YYCOPY(To, From, Count)		\
      do					\
	{					\
	  YYSIZE_T yyi;				\
	  for (yyi = 0; yyi < (Count); yyi++)	\
	    (To)[yyi] = (From)[yyi];		\
	}					\
      while (YYID (0))
#  endif
# endif

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
    do									\
      {									\
	YYSIZE_T yynewbytes;						\
	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
	Stack = &yyptr->Stack_alloc;					\
	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
	yyptr += yynewbytes / sizeof (*yyptr);				\
      }									\
    while (YYID (0))

#endif

/* YYFINAL -- State number of the termination state.  */
#define YYFINAL  2
/* YYLAST -- Last index in YYTABLE.  */
#define YYLAST   244

/* YYNTOKENS -- Number of terminals.  */
#define YYNTOKENS  80
/* YYNNTS -- Number of nonterminals.  */
#define YYNNTS  17
/* YYNRULES -- Number of rules.  */
#define YYNRULES  109
/* YYNRULES -- Number of states.  */
#define YYNSTATES  195

/* YYTRANSLATE(YYLEX) -- Bison symbol number corresponding to YYLEX.  */
#define YYUNDEFTOK  2
#define YYMAXUTOK   310

#define YYTRANSLATE(YYX)						\
  ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)

/* YYTRANSLATE[YYLEX] -- Bison symbol number corresponding to YYLEX.  */
static const yytype_uint8 yytranslate[] =
{
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,    74,     2,     2,    73,    75,
       2,     2,    71,     2,    59,     2,    76,    70,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,    63,
      62,     2,    64,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,    69,     2,     2,
       2,     2,     2,     2,    68,     2,     2,     2,     2,     2,
       2,     2,    67,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,    78,    65,     2,
       2,    57,     2,     2,    77,    61,    60,     2,    58,     2,
      79,     2,    56,     2,    66,     2,    72,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
       5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
      15,    16,    17,    18,    19,    20,    21,    22,    23,    24,
      25,    26,    27,    28,    29,    30,    31,    32,    33,    34,
      35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
      45,    46,    47,    48,    49,    50,    51,    52,    53,    54,
      55
};

#if YYDEBUG
/* YYPRHS[YYN] -- Index of the first RHS symbol of rule number YYN in
   YYRHS.  */
static const yytype_uint16 yyprhs[] =
{
       0,     0,     3,     6,     7,     9,    11,    13,    16,    19,
      22,    25,    28,    31,    36,    39,    42,    44,    46,    48,
      51,    53,    55,    57,    59,    61,    64,    69,    71,    75,
      78,    80,    82,    84,    86,    88,    90,    92,    94,    96,
      98,   100,   102,   104,   106,   109,   113,   115,   118,   121,
     124,   127,   129,   132,   135,   139,   142,   144,   147,   149,
     152,   154,   157,   159,   163,   165,   169,   172,   176,   179,
     182,   184,   186,   190,   196,   204,   214,   217,   220,   224,
     227,   230,   234,   237,   241,   245,   250,   256,   264,   268,
     273,   278,   283,   291,   296,   302,   307,   312,   318,   321,
     325,   329,   333,   337,   340,   345,   349,   352,   356,   359
};

/* YYRHS -- A `-1'-separated list of the rules' RHS.  */
static const yytype_int8 yyrhs[] =
{
      81,     0,    -1,    81,    82,    -1,    -1,    87,    -1,    90,
      -1,    91,    -1,    56,    89,    -1,    57,     3,    -1,    57,
       4,    -1,    58,     3,    -1,    58,     5,    -1,    58,     6,
      -1,    58,     5,    59,     5,    -1,    60,    83,    -1,    61,
      85,    -1,    62,    -1,    63,    -1,    64,    -1,    65,    89,
      -1,    66,    -1,    10,    -1,    11,    -1,    12,    -1,    67,
      -1,    68,    85,    -1,    69,     3,    70,     3,    -1,    17,
      -1,    13,    71,     3,    -1,    13,    14,    -1,    15,    -1,
      16,    -1,    18,    -1,    92,    -1,    96,    -1,     3,    -1,
       4,    -1,     5,    -1,     6,    -1,     3,    -1,     5,    -1,
      83,    -1,    84,    -1,    88,    -1,    87,    89,    -1,    87,
      72,    86,    -1,     7,    -1,    88,    73,    -1,    88,    74,
      -1,    88,    75,    -1,    88,    59,    -1,     3,    -1,    89,
       8,    -1,    89,     9,    -1,    89,    71,    85,    -1,    89,
      76,    -1,    77,    -1,    90,    89,    -1,    78,    -1,    91,
      89,    -1,    79,    -1,    79,    93,    -1,    94,    -1,    93,
      59,    94,    -1,    89,    -1,    94,    72,    86,    -1,    94,
       4,    -1,    94,    70,     3,    -1,    94,     8,    -1,    94,
       9,    -1,     3,    -1,    20,    -1,    23,    19,    95,    -1,
      23,    19,    95,    95,    95,    -1,    23,    19,    95,    95,
      95,    86,    86,    -1,    23,    19,    95,    95,    95,    86,
      86,    86,    86,    -1,    25,    19,    -1,    26,    19,    -1,
      26,    19,    19,    -1,    27,    21,    -1,    28,    21,    -1,
      29,    21,    86,    -1,    30,    86,    -1,    31,    83,    86,
      -1,    32,    86,    86,    -1,    32,    86,    86,    86,    -1,
      32,    86,    86,    86,    86,    -1,    32,    86,    86,    86,
      86,    86,    86,    -1,    34,    83,    21,    -1,    35,    83,
      21,    86,    -1,    36,    83,    21,    86,    -1,    37,    83,
      21,    86,    -1,    38,    83,    21,    86,    21,    86,    86,
      -1,    39,    83,    86,    86,    -1,    39,    83,    86,    86,
      86,    -1,    40,    83,    86,    86,    -1,    41,    83,    86,
      86,    -1,    42,    83,    21,    86,    86,    -1,    43,    83,
      -1,    44,    83,    86,    -1,    45,    83,    86,    -1,    46,
      83,    86,    -1,    47,    83,    86,    -1,    48,    83,    -1,
      49,    95,    86,    86,    -1,    51,    95,    86,    -1,    33,
      86,    -1,    52,    19,    19,    -1,    53,    83,    -1,    54,
      83,    -1
};

/* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
static const yytype_uint16 yyrline[] =
{
       0,   801,   801,   802,   806,   811,   815,   819,   823,   827,
     831,   835,   839,   844,   849,   853,   858,   862,   866,   870,
     875,   880,   884,   888,   893,   899,   903,   907,   912,   922,
     926,   930,   934,   939,   941,   946,   947,   951,   952,   956,
     957,   961,   962,   966,   967,   968,   972,   973,   974,   975,
     976,   980,   981,   982,   983,   984,   988,   989,   996,   997,
    1004,  1009,  1013,  1018,  1025,  1029,  1033,  1037,  1041,  1045,
    1052,  1053,  1057,  1061,  1065,  1069,  1074,  1078,  1082,  1087,
    1091,  1095,  1099,  1103,  1107,  1112,  1118,  1125,  1134,  1140,
    1146,  1152,  1158,  1164,  1170,  1176,  1182,  1188,  1194,  1200,
    1205,  1210,  1215,  1221,  1227,  1234,  1239,  1244,  1249,  1254
};
#endif

#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
   First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
static const char *const yytname[] =
{
  "$end", "error", "$undefined", "P_INTEGER", "S_INTEGER", "P_REAL",
  "S_REAL", "NOTE_P", "NOTE_T3", "NOTE_T5", "NEW_MARK", "GOTO_MARK",
  "ASSERT_MARK", "BLOCK", "BLK_ASSIGN", "BLK_INSERT", "FILE_INSERT",
  "ALTSTR", "LIBPATH_ADD", "XC_STR", "XC_ABSNOTE", "XC_MSECS", "SS_SEP",
  "SS_WAV", "SS_LOOP_WAV", "SS_PAT", "SS_SF2", "SS_SUSTAIN", "SS_ATTACK",
  "SS_VIBRATO", "SS_PORTAMENTO", "SS_CHANNEL", "SS_VOL", "SS_MASTER_VOL",
  "SS_EFF_DELAY", "SS_EFF_ECHO", "SS_EFF_COMB", "SS_EFF_ALLPASS",
  "SS_EFF_FLANGER", "SS_EFF_WOBBLE", "SS_EFF_SQWOBBLE", "SS_EFF_HFWOBBLE",
  "SS_EFF_FADER", "SS_EFF_REVERB", "SS_EFF_FOLDBACK", "SS_EFF_ATAN",
  "SS_EFF_DISTORT", "SS_EFF_OVERDRIVE", "SS_EFF_OFF", "SS_PITCH_STRETCH",
  "SS_TIME_STRETCH", "SS_PRINT_WAVE_TEMPO", "SONG_INFO", "MIDI_CHANNEL",
  "MIDI_PROGRAM", "MIDI_GENERIC", "'z'", "'o'", "'v'", "','", "'t'", "'s'",
  "'<'", "';'", "'>'", "'l'", "'|'", "'\\\\'", "'T'", "'M'", "'/'", "'*'",
  "'~'", "'&'", "'#'", "'\\''", "'.'", "'r'", "'k'", "'x'", "$accept",
  "script", "stmt", "integer", "real", "p_number", "number", "note",
  "note_pitch", "note_length", "rest", "back", "arp", "arp_list",
  "arp_note", "xc_absnote", "xc_cmd", 0
};
#endif

# ifdef YYPRINT
/* YYTOKNUM[YYLEX-NUM] -- Internal token number corresponding to
   token YYLEX-NUM.  */
static const yytype_uint16 yytoknum[] =
{
       0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
     265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
     275,   276,   277,   278,   279,   280,   281,   282,   283,   284,
     285,   286,   287,   288,   289,   290,   291,   292,   293,   294,
     295,   296,   297,   298,   299,   300,   301,   302,   303,   304,
     305,   306,   307,   308,   309,   310,   122,   111,   118,    44,
     116,   115,    60,    59,    62,   108,   124,    92,    84,    77,
      47,    42,   126,    38,    35,    39,    46,   114,   107,   120
};
# endif

/* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
static const yytype_uint8 yyr1[] =
{
       0,    80,    81,    81,    82,    82,    82,    82,    82,    82,
      82,    82,    82,    82,    82,    82,    82,    82,    82,    82,
      82,    82,    82,    82,    82,    82,    82,    82,    82,    82,
      82,    82,    82,    82,    82,    83,    83,    84,    84,    85,
      85,    86,    86,    87,    87,    87,    88,    88,    88,    88,
      88,    89,    89,    89,    89,    89,    90,    90,    91,    91,
      92,    92,    93,    93,    94,    94,    94,    94,    94,    94,
      95,    95,    96,    96,    96,    96,    96,    96,    96,    96,
      96,    96,    96,    96,    96,    96,    96,    96,    96,    96,
      96,    96,    96,    96,    96,    96,    96,    96,    96,    96,
      96,    96,    96,    96,    96,    96,    96,    96,    96,    96
};

/* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN.  */
static const yytype_uint8 yyr2[] =
{
       0,     2,     2,     0,     1,     1,     1,     2,     2,     2,
       2,     2,     2,     4,     2,     2,     1,     1,     1,     2,
       1,     1,     1,     1,     1,     2,     4,     1,     3,     2,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     2,     3,     1,     2,     2,     2,
       2,     1,     2,     2,     3,     2,     1,     2,     1,     2,
       1,     2,     1,     3,     1,     3,     2,     3,     2,     2,
       1,     1,     3,     5,     7,     9,     2,     2,     3,     2,
       2,     3,     2,     3,     3,     4,     5,     7,     3,     4,
       4,     4,     7,     4,     5,     4,     4,     5,     2,     3,
       3,     3,     3,     2,     4,     3,     2,     3,     2,     2
};

/* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
   STATE-NUM when YYTABLE doesn't specify something else to do.  Zero
   means the default is an error.  */
static const yytype_uint8 yydefact[] =
{
       3,     0,     1,    46,    21,    22,    23,     0,    30,    31,
      27,    32,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    16,    17,    18,
       0,    20,    24,     0,     0,    56,    58,    60,     2,     4,
      43,     5,     6,    33,    34,    29,     0,     0,    76,    77,
      79,    80,     0,    35,    36,    37,    38,    41,    42,    82,
       0,     0,   106,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    98,     0,     0,     0,     0,   103,    70,    71,
       0,     0,     0,   108,   109,    51,     7,     8,     9,    10,
      11,    12,    14,    39,    40,    15,    19,    25,     0,    64,
      61,    62,     0,    44,    50,    47,    48,    49,    57,    59,
      28,    72,    78,    81,    83,    84,    88,     0,     0,     0,
       0,     0,     0,     0,     0,    99,   100,   101,   102,     0,
     105,   107,    52,    53,     0,    55,     0,     0,     0,    66,
      68,    69,     0,     0,    45,     0,    85,    89,    90,    91,
       0,    93,    95,    96,     0,   104,    54,    13,    26,    63,
      67,    65,    73,    86,     0,    94,    97,     0,     0,     0,
      74,    87,    92,     0,    75
};

/* YYDEFGOTO[NTERM-NUM].  */
static const yytype_int8 yydefgoto[] =
{
      -1,     1,    58,    77,    78,   115,    79,    59,    60,   119,
      61,    62,    63,   120,   121,   100,    64
};

/* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
   STATE-NUM.  */
#define YYPACT_NINF -60
static const yytype_int16 yypact[] =
{
     -60,   165,   -60,   -60,   -60,   -60,   -60,    -8,   -60,   -60,
     -60,   -60,     3,     8,    16,     9,    10,    13,     7,    11,
       7,     7,    11,    11,    11,    11,    11,    11,    11,    11,
      11,    11,    11,    11,    11,    11,    11,     4,     4,    32,
      11,    11,    50,    15,    59,    11,    20,   -60,   -60,   -60,
      50,   -60,   -60,    20,    53,   -60,   -60,    50,   -60,     5,
      19,    50,    50,   -60,   -60,   -60,    54,     4,   -60,    39,
     -60,   -60,     7,   -60,   -60,   -60,   -60,   -60,   -60,   -60,
       7,     7,   -60,    45,    58,    62,    64,    65,     7,     7,
       7,    66,   -60,     7,     7,     7,     7,   -60,   -60,   -60,
       7,     7,    52,   -60,   -60,   -60,    -4,   -60,   -60,   -60,
      29,   -60,   -60,   -60,   -60,   -60,    -4,   -60,    21,    -4,
      30,    12,     7,    -4,   -60,   -60,   -60,   -60,    -4,    -4,
     -60,     4,   -60,   -60,   -60,     7,   -60,     7,     7,     7,
       7,     7,     7,     7,     7,   -60,   -60,   -60,   -60,     7,
     -60,   -60,   -60,   -60,    20,   -60,    85,    92,    50,   -60,
     -60,   -60,    94,     7,   -60,     4,     7,   -60,   -60,   -60,
      77,     7,   -60,   -60,     7,   -60,   -60,   -60,   -60,    12,
     -60,   -60,     7,     7,     7,   -60,   -60,     7,     7,     7,
       7,   -60,   -60,     7,   -60
};

/* YYPGOTO[NTERM-NUM].  */
static const yytype_int8 yypgoto[] =
{
     -60,   -60,   -60,    14,   -60,   -51,   -20,   -60,   -60,   -33,
     -60,   -60,   -60,   -60,   -59,   -35,   -60
};

/* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
   positive, shift that token.  If negative, reduce the rule which
   number is the opposite.  If zero, do what YYDEFACT says.
   If YYTABLE_NINF, syntax error.  */
#define YYTABLE_NINF -1
static const yytype_uint8 yytable[] =
{
      81,    82,   117,   101,   152,   153,    65,    98,   105,   106,
      73,    74,    75,    76,    73,    74,   159,   116,   107,   108,
     160,   161,    67,   113,    99,   114,   123,    68,   128,   129,
      70,    71,   131,    80,    72,    69,    83,    84,    85,    86,
      87,    88,    89,    90,    91,    92,    93,    94,    95,    96,
      97,   102,   133,   105,   103,   104,   118,   130,   132,   112,
     134,   135,   109,    66,   110,   111,   136,   154,   141,   142,
     143,   151,   155,   145,   146,   147,   148,   122,   124,   137,
     149,   150,   162,   138,   163,   139,   140,   144,   156,   158,
     177,   157,   125,   126,   127,   178,   165,   180,   184,   179,
       0,     0,   164,   176,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   166,     0,   167,   168,   169,
     170,   171,   172,   173,   174,     0,     0,     0,     0,   175,
     182,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   181,     0,     0,   183,     0,     0,     0,
       0,   185,     0,     0,   186,     0,     0,     0,     0,     0,
       0,     0,   187,   188,   189,     2,     0,   190,   191,   192,
     193,     0,     3,   194,     0,     4,     5,     6,     7,     0,
       8,     9,    10,    11,     0,     0,     0,     0,    12,     0,
      13,    14,    15,    16,    17,    18,    19,    20,    21,    22,
      23,    24,    25,    26,    27,    28,    29,    30,    31,    32,
      33,    34,    35,    36,    37,     0,    38,    39,    40,    41,
       0,    42,    43,    44,     0,    45,    46,    47,    48,    49,
      50,    51,    52,    53,    54,     0,     0,     0,     0,     0,
       0,     0,    55,    56,    57
};

static const yytype_int16 yycheck[] =
{
      20,    21,    53,    38,     8,     9,    14,     3,     3,    42,
       3,     4,     5,     6,     3,     4,     4,    50,     3,     4,
       8,     9,    19,     3,    20,     5,    59,    19,    61,    62,
      21,    21,    67,    19,    21,    19,    22,    23,    24,    25,
      26,    27,    28,    29,    30,    31,    32,    33,    34,    35,
      36,    19,    72,     3,    40,    41,     3,     3,    19,    45,
      80,    81,     3,    71,     5,     6,    21,    71,    88,    89,
      90,    19,    76,    93,    94,    95,    96,    72,    59,    21,
     100,   101,    70,    21,    72,    21,    21,    21,    59,    59,
       5,    70,    73,    74,    75,     3,   131,     3,    21,   158,
      -1,    -1,   122,   154,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   135,    -1,   137,   138,   139,
     140,   141,   142,   143,   144,    -1,    -1,    -1,    -1,   149,
     165,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   163,    -1,    -1,   166,    -1,    -1,    -1,
      -1,   171,    -1,    -1,   174,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   182,   183,   184,     0,    -1,   187,   188,   189,
     190,    -1,     7,   193,    -1,    10,    11,    12,    13,    -1,
      15,    16,    17,    18,    -1,    -1,    -1,    -1,    23,    -1,
      25,    26,    27,    28,    29,    30,    31,    32,    33,    34,
      35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
      45,    46,    47,    48,    49,    -1,    51,    52,    53,    54,
      -1,    56,    57,    58,    -1,    60,    61,    62,    63,    64,
      65,    66,    67,    68,    69,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    77,    78,    79
};

/* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
   symbol of state STATE-NUM.  */
static const yytype_uint8 yystos[] =
{
       0,    81,     0,     7,    10,    11,    12,    13,    15,    16,
      17,    18,    23,    25,    26,    27,    28,    29,    30,    31,
      32,    33,    34,    35,    36,    37,    38,    39,    40,    41,
      42,    43,    44,    45,    46,    47,    48,    49,    51,    52,
      53,    54,    56,    57,    58,    60,    61,    62,    63,    64,
      65,    66,    67,    68,    69,    77,    78,    79,    82,    87,
      88,    90,    91,    92,    96,    14,    71,    19,    19,    19,
      21,    21,    21,     3,     4,     5,     6,    83,    84,    86,
      83,    86,    86,    83,    83,    83,    83,    83,    83,    83,
      83,    83,    83,    83,    83,    83,    83,    83,     3,    20,
      95,    95,    19,    83,    83,     3,    89,     3,     4,     3,
       5,     6,    83,     3,     5,    85,    89,    85,     3,    89,
      93,    94,    72,    89,    59,    73,    74,    75,    89,    89,
       3,    95,    19,    86,    86,    86,    21,    21,    21,    21,
      21,    86,    86,    86,    21,    86,    86,    86,    86,    86,
      86,    19,     8,     9,    71,    76,    59,    70,    59,     4,
       8,     9,    70,    72,    86,    95,    86,    86,    86,    86,
      86,    86,    86,    86,    86,    86,    85,     5,     3,    94,
       3,    86,    95,    86,    21,    86,    86,    86,    86,    86,
      86,    86,    86,    86,    86
};

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		(-2)
#define YYEOF		0

#define YYACCEPT	goto yyacceptlab
#define YYABORT		goto yyabortlab
#define YYERROR		goto yyerrorlab


/* Like YYERROR except do call yyerror.  This remains here temporarily
   to ease the transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */

#define YYFAIL		goto yyerrlab

#define YYRECOVERING()  (!!yyerrstatus)

#define YYBACKUP(Token, Value)					\
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    {								\
      yychar = (Token);						\
      yylval = (Value);						\
      yytoken = YYTRANSLATE (yychar);				\
      YYPOPSTACK (1);						\
      goto yybackup;						\
    }								\
  else								\
    {								\
      yyerror (YY_("syntax error: cannot back up")); \
      YYERROR;							\
    }								\
while (YYID (0))


#define YYTERROR	1
#define YYERRCODE	256


/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
   If N is 0, then set CURRENT to the empty location which ends
   the previous symbol: RHS[0] (always defined).  */

#define YYRHSLOC(Rhs, K) ((Rhs)[K])
#ifndef YYLLOC_DEFAULT
# define YYLLOC_DEFAULT(Current, Rhs, N)				\
    do									\
      if (YYID (N))                                                    \
	{								\
	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
	}								\
      else								\
	{								\
	  (Current).first_line   = (Current).last_line   =		\
	    YYRHSLOC (Rhs, 0).last_line;				\
	  (Current).first_column = (Current).last_column =		\
	    YYRHSLOC (Rhs, 0).last_column;				\
	}								\
    while (YYID (0))
#endif


/* YY_LOCATION_PRINT -- Print the location on the stream.
   This macro was not mandated originally: define only if we know
   we won't break user code: when these are the locations we know.  */

#ifndef YY_LOCATION_PRINT
# if YYLTYPE_IS_TRIVIAL
#  define YY_LOCATION_PRINT(File, Loc)			\
     fprintf (File, "%d.%d-%d.%d",			\
	      (Loc).first_line, (Loc).first_column,	\
	      (Loc).last_line,  (Loc).last_column)
# else
#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
# endif
#endif


/* YYLEX -- calling `yylex' with the right arguments.  */

#ifdef YYLEX_PARAM
# define YYLEX yylex (YYLEX_PARAM)
#else
# define YYLEX yylex ()
#endif

/* Enable debugging if requested.  */
#if YYDEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)			\
do {						\
  if (yydebug)					\
    YYFPRINTF Args;				\
} while (YYID (0))

# define YY_SYMBOL_PRINT(Title, Type, Value, Location)			  \
do {									  \
  if (yydebug)								  \
    {									  \
      YYFPRINTF (stderr, "%s ", Title);					  \
      yy_symbol_print (stderr,						  \
		  Type, Value); \
      YYFPRINTF (stderr, "\n");						  \
    }									  \
} while (YYID (0))


/*--------------------------------.
| Print this symbol on YYOUTPUT.  |
`--------------------------------*/

/*ARGSUSED*/
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
#else
static void
yy_symbol_value_print (yyoutput, yytype, yyvaluep)
    FILE *yyoutput;
    int yytype;
    YYSTYPE const * const yyvaluep;
#endif
{
  if (!yyvaluep)
    return;
# ifdef YYPRINT
  if (yytype < YYNTOKENS)
    YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
# else
  YYUSE (yyoutput);
# endif
  switch (yytype)
    {
      default:
	break;
    }
}


/*--------------------------------.
| Print this symbol on YYOUTPUT.  |
`--------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
#else
static void
yy_symbol_print (yyoutput, yytype, yyvaluep)
    FILE *yyoutput;
    int yytype;
    YYSTYPE const * const yyvaluep;
#endif
{
  if (yytype < YYNTOKENS)
    YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
  else
    YYFPRINTF (yyoutput, "nterm %s (", yytname[yytype]);

  yy_symbol_value_print (yyoutput, yytype, yyvaluep);
  YYFPRINTF (yyoutput, ")");
}

/*------------------------------------------------------------------.
| yy_stack_print -- Print the state stack from its BOTTOM up to its |
| TOP (included).                                                   |
`------------------------------------------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
#else
static void
yy_stack_print (yybottom, yytop)
    yytype_int16 *yybottom;
    yytype_int16 *yytop;
#endif
{
  YYFPRINTF (stderr, "Stack now");
  for (; yybottom <= yytop; yybottom++)
    {
      int yybot = *yybottom;
      YYFPRINTF (stderr, " %d", yybot);
    }
  YYFPRINTF (stderr, "\n");
}

# define YY_STACK_PRINT(Bottom, Top)				\
do {								\
  if (yydebug)							\
    yy_stack_print ((Bottom), (Top));				\
} while (YYID (0))


/*------------------------------------------------.
| Report that the YYRULE is going to be reduced.  |
`------------------------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_reduce_print (YYSTYPE *yyvsp, int yyrule)
#else
static void
yy_reduce_print (yyvsp, yyrule)
    YYSTYPE *yyvsp;
    int yyrule;
#endif
{
  int yynrhs = yyr2[yyrule];
  int yyi;
  unsigned long int yylno = yyrline[yyrule];
  YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu):\n",
	     yyrule - 1, yylno);
  /* The symbols being reduced.  */
  for (yyi = 0; yyi < yynrhs; yyi++)
    {
      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
      yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
		       &(yyvsp[(yyi + 1) - (yynrhs)])
		       		       );
      YYFPRINTF (stderr, "\n");
    }
}

# define YY_REDUCE_PRINT(Rule)		\
do {					\
  if (yydebug)				\
    yy_reduce_print (yyvsp, Rule); \
} while (YYID (0))

/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !YYDEBUG */
# define YYDPRINTF(Args)
# define YY_SYMBOL_PRINT(Title, Type, Value, Location)
# define YY_STACK_PRINT(Bottom, Top)
# define YY_REDUCE_PRINT(Rule)
#endif /* !YYDEBUG */


/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef	YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif



#if YYERROR_VERBOSE

# ifndef yystrlen
#  if defined __GLIBC__ && defined _STRING_H
#   define yystrlen strlen
#  else
/* Return the length of YYSTR.  */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static YYSIZE_T
yystrlen (const char *yystr)
#else
static YYSIZE_T
yystrlen (yystr)
    const char *yystr;
#endif
{
  YYSIZE_T yylen;
  for (yylen = 0; yystr[yylen]; yylen++)
    continue;
  return yylen;
}
#  endif
# endif

# ifndef yystpcpy
#  if defined __GLIBC__ && defined _STRING_H && defined _GNU_SOURCE
#   define yystpcpy stpcpy
#  else
/* Copy YYSRC to YYDEST, returning the address of the terminating '\0' in
   YYDEST.  */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static char *
yystpcpy (char *yydest, const char *yysrc)
#else
static char *
yystpcpy (yydest, yysrc)
    char *yydest;
    const char *yysrc;
#endif
{
  char *yyd = yydest;
  const char *yys = yysrc;

  while ((*yyd++ = *yys++) != '\0')
    continue;

  return yyd - 1;
}
#  endif
# endif

# ifndef yytnamerr
/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
   quotes and backslashes, so that it's suitable for yyerror.  The
   heuristic is that double-quoting is unnecessary unless the string
   contains an apostrophe, a comma, or backslash (other than
   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
   null, do not copy; instead, return the length of what the result
   would have been.  */
static YYSIZE_T
yytnamerr (char *yyres, const char *yystr)
{
  if (*yystr == '"')
    {
      YYSIZE_T yyn = 0;
      char const *yyp = yystr;

      for (;;)
	switch (*++yyp)
	  {
	  case '\'':
	  case ',':
	    goto do_not_strip_quotes;

	  case '\\':
	    if (*++yyp != '\\')
	      goto do_not_strip_quotes;
	    /* Fall through.  */
	  default:
	    if (yyres)
	      yyres[yyn] = *yyp;
	    yyn++;
	    break;

	  case '"':
	    if (yyres)
	      yyres[yyn] = '\0';
	    return yyn;
	  }
    do_not_strip_quotes: ;
    }

  if (! yyres)
    return yystrlen (yystr);

  return yystpcpy (yyres, yystr) - yyres;
}
# endif

/* Copy into YYRESULT an error message about the unexpected token
   YYCHAR while in state YYSTATE.  Return the number of bytes copied,
   including the terminating null byte.  If YYRESULT is null, do not
   copy anything; just return the number of bytes that would be
   copied.  As a special case, return 0 if an ordinary "syntax error"
   message will do.  Return YYSIZE_MAXIMUM if overflow occurs during
   size calculation.  */
static YYSIZE_T
yysyntax_error (char *yyresult, int yystate, int yychar)
{
  int yyn = yypact[yystate];

  if (! (YYPACT_NINF < yyn && yyn <= YYLAST))
    return 0;
  else
    {
      int yytype = YYTRANSLATE (yychar);
      YYSIZE_T yysize0 = yytnamerr (0, yytname[yytype]);
      YYSIZE_T yysize = yysize0;
      YYSIZE_T yysize1;
      int yysize_overflow = 0;
      enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
      char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
      int yyx;

# if 0
      /* This is so xgettext sees the translatable formats that are
	 constructed on the fly.  */
      YY_("syntax error, unexpected %s");
      YY_("syntax error, unexpected %s, expecting %s");
      YY_("syntax error, unexpected %s, expecting %s or %s");
      YY_("syntax error, unexpected %s, expecting %s or %s or %s");
      YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s");
# endif
      char *yyfmt;
      char const *yyf;
      static char const yyunexpected[] = "syntax error, unexpected %s";
      static char const yyexpecting[] = ", expecting %s";
      static char const yyor[] = " or %s";
      char yyformat[sizeof yyunexpected
		    + sizeof yyexpecting - 1
		    + ((YYERROR_VERBOSE_ARGS_MAXIMUM - 2)
		       * (sizeof yyor - 1))];
      char const *yyprefix = yyexpecting;

      /* Start YYX at -YYN if negative to avoid negative indexes in
	 YYCHECK.  */
      int yyxbegin = yyn < 0 ? -yyn : 0;

      /* Stay within bounds of both yycheck and yytname.  */
      int yychecklim = YYLAST - yyn + 1;
      int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
      int yycount = 1;

      yyarg[0] = yytname[yytype];
      yyfmt = yystpcpy (yyformat, yyunexpected);

      for (yyx = yyxbegin; yyx < yyxend; ++yyx)
	if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
	  {
	    if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
	      {
		yycount = 1;
		yysize = yysize0;
		yyformat[sizeof yyunexpected - 1] = '\0';
		break;
	      }
	    yyarg[yycount++] = yytname[yyx];
	    yysize1 = yysize + yytnamerr (0, yytname[yyx]);
	    yysize_overflow |= (yysize1 < yysize);
	    yysize = yysize1;
	    yyfmt = yystpcpy (yyfmt, yyprefix);
	    yyprefix = yyor;
	  }

      yyf = YY_(yyformat);
      yysize1 = yysize + yystrlen (yyf);
      yysize_overflow |= (yysize1 < yysize);
      yysize = yysize1;

      if (yysize_overflow)
	return YYSIZE_MAXIMUM;

      if (yyresult)
	{
	  /* Avoid sprintf, as that infringes on the user's name space.
	     Don't have undefined behavior even if the translation
	     produced a string with the wrong number of "%s"s.  */
	  char *yyp = yyresult;
	  int yyi = 0;
	  while ((*yyp = *yyf) != '\0')
	    {
	      if (*yyp == '%' && yyf[1] == 's' && yyi < yycount)
		{
		  yyp += yytnamerr (yyp, yyarg[yyi++]);
		  yyf += 2;
		}
	      else
		{
		  yyp++;
		  yyf++;
		}
	    }
	}
      return yysize;
    }
}
#endif /* YYERROR_VERBOSE */


/*-----------------------------------------------.
| Release the memory associated to this symbol.  |
`-----------------------------------------------*/

/*ARGSUSED*/
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
#else
static void
yydestruct (yymsg, yytype, yyvaluep)
    const char *yymsg;
    int yytype;
    YYSTYPE *yyvaluep;
#endif
{
  YYUSE (yyvaluep);

  if (!yymsg)
    yymsg = "Deleting";
  YY_SYMBOL_PRINT (yymsg, yytype, yyvaluep, yylocationp);

  switch (yytype)
    {

      default:
	break;
    }
}

/* Prevent warnings from -Wmissing-prototypes.  */
#ifdef YYPARSE_PARAM
#if defined __STDC__ || defined __cplusplus
int yyparse (void *YYPARSE_PARAM);
#else
int yyparse ();
#endif
#else /* ! YYPARSE_PARAM */
#if defined __STDC__ || defined __cplusplus
int yyparse (void);
#else
int yyparse ();
#endif
#endif /* ! YYPARSE_PARAM */


/* The lookahead symbol.  */
int yychar;

/* The semantic value of the lookahead symbol.  */
YYSTYPE yylval;

/* Number of syntax errors so far.  */
int yynerrs;



/*-------------------------.
| yyparse or yypush_parse.  |
`-------------------------*/

#ifdef YYPARSE_PARAM
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
int
yyparse (void *YYPARSE_PARAM)
#else
int
yyparse (YYPARSE_PARAM)
    void *YYPARSE_PARAM;
#endif
#else /* ! YYPARSE_PARAM */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
int
yyparse (void)
#else
int
yyparse ()

#endif
#endif
{


    int yystate;
    /* Number of tokens to shift before error messages enabled.  */
    int yyerrstatus;

    /* The stacks and their tools:
       `yyss': related to states.
       `yyvs': related to semantic values.

       Refer to the stacks thru separate pointers, to allow yyoverflow
       to reallocate them elsewhere.  */

    /* The state stack.  */
    yytype_int16 yyssa[YYINITDEPTH];
    yytype_int16 *yyss;
    yytype_int16 *yyssp;

    /* The semantic value stack.  */
    YYSTYPE yyvsa[YYINITDEPTH];
    YYSTYPE *yyvs;
    YYSTYPE *yyvsp;

    YYSIZE_T yystacksize;

  int yyn;
  int yyresult;
  /* Lookahead token as an internal (translated) token number.  */
  int yytoken;
  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;

#if YYERROR_VERBOSE
  /* Buffer for error messages, and its allocated size.  */
  char yymsgbuf[128];
  char *yymsg = yymsgbuf;
  YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
#endif

#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))

  /* The number of symbols on the RHS of the reduced rule.
     Keep to zero when no symbol should be popped.  */
  int yylen = 0;

  yytoken = 0;
  yyss = yyssa;
  yyvs = yyvsa;
  yystacksize = YYINITDEPTH;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY; /* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */
  yyssp = yyss;
  yyvsp = yyvs;

  goto yysetstate;

/*------------------------------------------------------------.
| yynewstate -- Push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
 yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed.  So pushing a state here evens the stacks.  */
  yyssp++;

 yysetstate:
  *yyssp = yystate;

  if (yyss + yystacksize - 1 <= yyssp)
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYSIZE_T yysize = yyssp - yyss + 1;

#ifdef yyoverflow
      {
	/* Give user a chance to reallocate the stack.  Use copies of
	   these so that the &'s don't force the real ones into
	   memory.  */
	YYSTYPE *yyvs1 = yyvs;
	yytype_int16 *yyss1 = yyss;

	/* Each stack pointer address is followed by the size of the
	   data in use in that stack, in bytes.  This used to be a
	   conditional around just the two extra args, but that might
	   be undefined if yyoverflow is a macro.  */
	yyoverflow (YY_("memory exhausted"),
		    &yyss1, yysize * sizeof (*yyssp),
		    &yyvs1, yysize * sizeof (*yyvsp),
		    &yystacksize);

	yyss = yyss1;
	yyvs = yyvs1;
      }
#else /* no yyoverflow */
# ifndef YYSTACK_RELOCATE
      goto yyexhaustedlab;
# else
      /* Extend the stack our own way.  */
      if (YYMAXDEPTH <= yystacksize)
	goto yyexhaustedlab;
      yystacksize *= 2;
      if (YYMAXDEPTH < yystacksize)
	yystacksize = YYMAXDEPTH;

      {
	yytype_int16 *yyss1 = yyss;
	union yyalloc *yyptr =
	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
	if (! yyptr)
	  goto yyexhaustedlab;
	YYSTACK_RELOCATE (yyss_alloc, yyss);
	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
#  undef YYSTACK_RELOCATE
	if (yyss1 != yyssa)
	  YYSTACK_FREE (yyss1);
      }
# endif
#endif /* no yyoverflow */

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;

      YYDPRINTF ((stderr, "Stack size increased to %lu\n",
		  (unsigned long int) yystacksize));

      if (yyss + yystacksize - 1 <= yyssp)
	YYABORT;
    }

  YYDPRINTF ((stderr, "Entering state %d\n", yystate));

  if (yystate == YYFINAL)
    YYACCEPT;

  goto yybackup;

/*-----------.
| yybackup.  |
`-----------*/
yybackup:

  /* Do appropriate processing given the current state.  Read a
     lookahead token if we need one and don't already have one.  */

  /* First try to decide what to do without reference to lookahead token.  */
  yyn = yypact[yystate];
  if (yyn == YYPACT_NINF)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
  if (yychar == YYEMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token: "));
      yychar = YYLEX;
    }

  if (yychar <= YYEOF)
    {
      yychar = yytoken = YYEOF;
      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else
    {
      yytoken = YYTRANSLATE (yychar);
      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
    }

  /* If the proper action on seeing token YYTOKEN is to reduce or to
     detect an error, take that action.  */
  yyn += yytoken;
  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
    goto yydefault;
  yyn = yytable[yyn];
  if (yyn <= 0)
    {
      if (yyn == 0 || yyn == YYTABLE_NINF)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  /* Shift the lookahead token.  */
  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);

  /* Discard the shifted token.  */
  yychar = YYEMPTY;

  yystate = yyn;
  *++yyvsp = yylval;

  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- Do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     `$$ = $1'.

     Otherwise, the following line sets YYVAL to garbage.
     This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];


  YY_REDUCE_PRINT (yyn);
  switch (yyn)
    {
        case 2:

/* Line 1455 of yacc.c  */
#line 801 "compiler.y"
    { ; }
    break;

  case 4:

/* Line 1455 of yacc.c  */
#line 806 "compiler.y"
    {
					/* note event */
					add_note_event((yyvsp[(1) - (1)].i));
					forward(length);
				}
    break;

  case 5:

/* Line 1455 of yacc.c  */
#line 811 "compiler.y"
    {
					/* rest */
					forward(length);
				}
    break;

  case 6:

/* Line 1455 of yacc.c  */
#line 815 "compiler.y"
    {
					/* back */
					add_back_event();
				}
    break;

  case 7:

/* Line 1455 of yacc.c  */
#line 819 "compiler.y"
    {
					/* zero note */
					length = (yyvsp[(2) - (2)].d);
				}
    break;

  case 8:

/* Line 1455 of yacc.c  */
#line 823 "compiler.y"
    {
					/* absolute octave */
					octave = (yyvsp[(2) - (2)].i);
				}
    break;

  case 9:

/* Line 1455 of yacc.c  */
#line 827 "compiler.y"
    {
					/* relative octave */
					octave += (yyvsp[(2) - (2)].i);
				}
    break;

  case 10:

/* Line 1455 of yacc.c  */
#line 831 "compiler.y"
    {
					/* absolute volume */
					volume_from = volume_to = (yyvsp[(2) - (2)].i);
				}
    break;

  case 11:

/* Line 1455 of yacc.c  */
#line 835 "compiler.y"
    {
					/* absolute volume */
					volume_from = volume_to = (yyvsp[(2) - (2)].d);
				}
    break;

  case 12:

/* Line 1455 of yacc.c  */
#line 839 "compiler.y"
    {
					/* relative volume */
					volume_from += (yyvsp[(2) - (2)].d);
					volume_to += (yyvsp[(2) - (2)].d);
				}
    break;

  case 13:

/* Line 1455 of yacc.c  */
#line 844 "compiler.y"
    {
					/* absolute volume ranges */
					volume_from = (yyvsp[(2) - (4)].d);
					volume_to = (yyvsp[(4) - (4)].d);
				}
    break;

  case 14:

/* Line 1455 of yacc.c  */
#line 849 "compiler.y"
    {
					/* transpose */
					transpose = (yyvsp[(2) - (2)].i);
				}
    break;

  case 15:

/* Line 1455 of yacc.c  */
#line 853 "compiler.y"
    {
					/* staccato */
					staccato = (yyvsp[(2) - (2)].d);
				}
    break;

  case 16:

/* Line 1455 of yacc.c  */
#line 858 "compiler.y"
    {
					/* start of group */
					push_group();
				}
    break;

  case 17:

/* Line 1455 of yacc.c  */
#line 862 "compiler.y"
    {
					/* group delimiter */
					next_group_part();
				}
    break;

  case 18:

/* Line 1455 of yacc.c  */
#line 866 "compiler.y"
    {
					/* end of group */
					pop_group();
				}
    break;

  case 19:

/* Line 1455 of yacc.c  */
#line 870 "compiler.y"
    {
					/* glissando */
					gliss = (yyvsp[(2) - (2)].d);
				}
    break;

  case 20:

/* Line 1455 of yacc.c  */
#line 875 "compiler.y"
    {
					/* measure mark event */
					add_measure_event();
				}
    break;

  case 21:

/* Line 1455 of yacc.c  */
#line 880 "compiler.y"
    {
					/* add new mark */
					add_mark((yyvsp[(1) - (1)].p));
				}
    break;

  case 22:

/* Line 1455 of yacc.c  */
#line 884 "compiler.y"
    {
					/* go to mark */
					find_mark((yyvsp[(1) - (1)].p), 1);
				}
    break;

  case 23:

/* Line 1455 of yacc.c  */
#line 888 "compiler.y"
    {
					/* assert mark */
					find_mark((yyvsp[(1) - (1)].p), 0);
				}
    break;

  case 24:

/* Line 1455 of yacc.c  */
#line 893 "compiler.y"
    {
					/* new track */

					track++;
					init_track();
				}
    break;

  case 25:

/* Line 1455 of yacc.c  */
#line 899 "compiler.y"
    {
					/* tempo setting */
					add_tempo_event(-1, (yyvsp[(2) - (2)].d));
				}
    break;

  case 26:

/* Line 1455 of yacc.c  */
#line 903 "compiler.y"
    {
					/* meter (time signature) setting */
					add_meter_event(-1, (yyvsp[(2) - (4)].i), (yyvsp[(4) - (4)].i));
				}
    break;

  case 27:

/* Line 1455 of yacc.c  */
#line 907 "compiler.y"
    {
					/* alteration string */
					set_alteration((yyvsp[(1) - (1)].p));
				}
    break;

  case 28:

/* Line 1455 of yacc.c  */
#line 912 "compiler.y"
    {
					/* repeat block */
					int n;

					/* store the block as <TMP> */
					set_block("<TMP>", (yyvsp[(1) - (3)].p));

					for(n = 0;n < (yyvsp[(3) - (3)].i);n++)
						insert_block("<TMP>");
				}
    break;

  case 29:

/* Line 1455 of yacc.c  */
#line 922 "compiler.y"
    {
					/* assign block */
					set_block((yyvsp[(2) - (2)].p), (yyvsp[(1) - (2)].p));
				}
    break;

  case 30:

/* Line 1455 of yacc.c  */
#line 926 "compiler.y"
    {
					/* insert block */
					insert_block((yyvsp[(1) - (1)].p));
				}
    break;

  case 31:

/* Line 1455 of yacc.c  */
#line 930 "compiler.y"
    {
					/* insert file */
					insert_file((yyvsp[(1) - (1)].p));
				}
    break;

  case 32:

/* Line 1455 of yacc.c  */
#line 934 "compiler.y"
    {
					/* add library path */
					libpath_add((yyvsp[(1) - (1)].p), 0);
				}
    break;

  case 33:

/* Line 1455 of yacc.c  */
#line 939 "compiler.y"
    { ; }
    break;

  case 34:

/* Line 1455 of yacc.c  */
#line 941 "compiler.y"
    { ; }
    break;

  case 35:

/* Line 1455 of yacc.c  */
#line 946 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (1)].i); }
    break;

  case 36:

/* Line 1455 of yacc.c  */
#line 947 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (1)].i); }
    break;

  case 37:

/* Line 1455 of yacc.c  */
#line 951 "compiler.y"
    { (yyval.d) = (yyvsp[(1) - (1)].d); }
    break;

  case 38:

/* Line 1455 of yacc.c  */
#line 952 "compiler.y"
    { (yyval.d) = (yyvsp[(1) - (1)].d); }
    break;

  case 39:

/* Line 1455 of yacc.c  */
#line 956 "compiler.y"
    { (yyval.d) = (double) (yyvsp[(1) - (1)].i); }
    break;

  case 40:

/* Line 1455 of yacc.c  */
#line 957 "compiler.y"
    { (yyval.d) = (yyvsp[(1) - (1)].d); }
    break;

  case 41:

/* Line 1455 of yacc.c  */
#line 961 "compiler.y"
    { (yyval.d) = (double) (yyvsp[(1) - (1)].i); }
    break;

  case 42:

/* Line 1455 of yacc.c  */
#line 962 "compiler.y"
    { (yyval.d) = (yyvsp[(1) - (1)].d); }
    break;

  case 43:

/* Line 1455 of yacc.c  */
#line 966 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (1)].i); }
    break;

  case 44:

/* Line 1455 of yacc.c  */
#line 967 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (2)].i); length=(yyvsp[(2) - (2)].d); }
    break;

  case 45:

/* Line 1455 of yacc.c  */
#line 968 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (3)].i); DEBUGF("imm volume: %lf\n", (yyvsp[(3) - (3)].d)); }
    break;

  case 46:

/* Line 1455 of yacc.c  */
#line 972 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (1)].i); }
    break;

  case 47:

/* Line 1455 of yacc.c  */
#line 973 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (2)].i) - 1; }
    break;

  case 48:

/* Line 1455 of yacc.c  */
#line 974 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (2)].i) + 1; }
    break;

  case 49:

/* Line 1455 of yacc.c  */
#line 975 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (2)].i) + 12; }
    break;

  case 50:

/* Line 1455 of yacc.c  */
#line 976 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (2)].i) - 12; }
    break;

  case 51:

/* Line 1455 of yacc.c  */
#line 980 "compiler.y"
    { (yyval.d) = 1 / (double) (yyvsp[(1) - (1)].i); }
    break;

  case 52:

/* Line 1455 of yacc.c  */
#line 981 "compiler.y"
    { (yyval.d) = (yyvsp[(1) - (2)].d) * 2.0 / 3.0; }
    break;

  case 53:

/* Line 1455 of yacc.c  */
#line 982 "compiler.y"
    { (yyval.d) = (yyvsp[(1) - (2)].d) * 4.0 / 5.0; }
    break;

  case 54:

/* Line 1455 of yacc.c  */
#line 983 "compiler.y"
    { (yyval.d) = (yyvsp[(1) - (3)].d) * (yyvsp[(3) - (3)].d); }
    break;

  case 55:

/* Line 1455 of yacc.c  */
#line 984 "compiler.y"
    { (yyval.d) = (yyvsp[(1) - (2)].d) * 1.5; }
    break;

  case 56:

/* Line 1455 of yacc.c  */
#line 988 "compiler.y"
    { ; }
    break;

  case 57:

/* Line 1455 of yacc.c  */
#line 989 "compiler.y"
    {
					/* rest with length */
					length = (yyvsp[(2) - (2)].d);
				}
    break;

  case 58:

/* Line 1455 of yacc.c  */
#line 996 "compiler.y"
    { ; }
    break;

  case 59:

/* Line 1455 of yacc.c  */
#line 997 "compiler.y"
    {
					/* back with length */
					length = (yyvsp[(2) - (2)].d);
				}
    break;

  case 60:

/* Line 1455 of yacc.c  */
#line 1004 "compiler.y"
    {
					/* empty arpeggiator */
					n_arps = 0;
					arp_default();
				}
    break;

  case 61:

/* Line 1455 of yacc.c  */
#line 1009 "compiler.y"
    { ; }
    break;

  case 62:

/* Line 1455 of yacc.c  */
#line 1013 "compiler.y"
    {
					/* first arpeggiator note */
					n_arps = 0;
					add_arp();
				}
    break;

  case 63:

/* Line 1455 of yacc.c  */
#line 1018 "compiler.y"
    {
					/* rest of arpeggiator notes */
					add_arp();
				}
    break;

  case 64:

/* Line 1455 of yacc.c  */
#line 1025 "compiler.y"
    {
					/* arpeggiator delay */
					arp_delay = (yyvsp[(1) - (1)].d);
				}
    break;

  case 65:

/* Line 1455 of yacc.c  */
#line 1029 "compiler.y"
    {
					/* arpeggiator volume */
					arp_volume = (yyvsp[(3) - (3)].d);
				}
    break;

  case 66:

/* Line 1455 of yacc.c  */
#line 1033 "compiler.y"
    {
					/* arpeggiator transpose */
					arp_transpose = (yyvsp[(2) - (2)].i);
				}
    break;

  case 67:

/* Line 1455 of yacc.c  */
#line 1037 "compiler.y"
    {
					/* arpeggiator track offset */
					arp_track_off = (yyvsp[(3) - (3)].i);
				}
    break;

  case 68:

/* Line 1455 of yacc.c  */
#line 1041 "compiler.y"
    {
					/* HACK: /3 */
					arp_track_off = 3;
				}
    break;

  case 69:

/* Line 1455 of yacc.c  */
#line 1045 "compiler.y"
    {
					/* HACK: /5 */
					arp_track_off = 5;
				}
    break;

  case 70:

/* Line 1455 of yacc.c  */
#line 1052 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (1)].i); }
    break;

  case 71:

/* Line 1455 of yacc.c  */
#line 1053 "compiler.y"
    { (yyval.i) = (yyvsp[(1) - (1)].i); }
    break;

  case 72:

/* Line 1455 of yacc.c  */
#line 1057 "compiler.y"
    {
					/* load .wav file with just one note */
					add_ss_wav_event((yyvsp[(2) - (3)].p), (yyvsp[(3) - (3)].i), (yyvsp[(3) - (3)].i), (yyvsp[(3) - (3)].i), -1.0, -1.0, 0, 0);
				}
    break;

  case 73:

/* Line 1455 of yacc.c  */
#line 1061 "compiler.y"
    {
					/* load .wav file */
					add_ss_wav_event((yyvsp[(2) - (5)].p), (yyvsp[(3) - (5)].i), (yyvsp[(4) - (5)].i), (yyvsp[(5) - (5)].i), -1.0, -1.0, 0, 0);
				}
    break;

  case 74:

/* Line 1455 of yacc.c  */
#line 1065 "compiler.y"
    {
					/* load .wav file, with loop boundaries */
					add_ss_wav_event((yyvsp[(2) - (7)].p), (yyvsp[(3) - (7)].i), (yyvsp[(4) - (7)].i), (yyvsp[(5) - (7)].i), (yyvsp[(6) - (7)].d), (yyvsp[(7) - (7)].d), 0, 0);
				}
    break;

  case 75:

/* Line 1455 of yacc.c  */
#line 1069 "compiler.y"
    {
					/* load .wav file, with loop boundaries,
					   first channel and skip channels */
					add_ss_wav_event((yyvsp[(2) - (9)].p), (yyvsp[(3) - (9)].i), (yyvsp[(4) - (9)].i), (yyvsp[(5) - (9)].i), (yyvsp[(6) - (9)].d), (yyvsp[(7) - (9)].d), (yyvsp[(8) - (9)].d), (yyvsp[(9) - (9)].d));
				}
    break;

  case 76:

/* Line 1455 of yacc.c  */
#line 1074 "compiler.y"
    {
					/* load .pat file */
					add_ss_pat_event((yyvsp[(2) - (2)].p));
				}
    break;

  case 77:

/* Line 1455 of yacc.c  */
#line 1078 "compiler.y"
    {
					/* list instruments in .sf2 file */
					add_ss_sf2_event((yyvsp[(2) - (2)].p), NULL);
				}
    break;

  case 78:

/* Line 1455 of yacc.c  */
#line 1082 "compiler.y"
    {
					/* load instrument from .sf2 file */
					add_ss_sf2_event((yyvsp[(2) - (3)].p), (yyvsp[(3) - (3)].p));
				}
    break;

  case 79:

/* Line 1455 of yacc.c  */
#line 1087 "compiler.y"
    {
					/* sets sustain */
					add_ss_sustain_event((yyvsp[(2) - (2)].d));
				}
    break;

  case 80:

/* Line 1455 of yacc.c  */
#line 1091 "compiler.y"
    {
					/* sets attack */
					add_ss_attack_event((yyvsp[(2) - (2)].d));
				}
    break;

  case 81:

/* Line 1455 of yacc.c  */
#line 1095 "compiler.y"
    {
					/* sets vibrato */
					add_ss_vibrato_event((yyvsp[(2) - (3)].d), (yyvsp[(3) - (3)].d));
				}
    break;

  case 82:

/* Line 1455 of yacc.c  */
#line 1099 "compiler.y"
    {
					/* sets portamento */
					add_ss_portamento_event((yyvsp[(2) - (2)].d));
				}
    break;

  case 83:

/* Line 1455 of yacc.c  */
#line 1103 "compiler.y"
    {
					/* sets volume for a channel */
					add_ss_channel_event((yyvsp[(2) - (3)].i), (yyvsp[(3) - (3)].d));
				}
    break;

  case 84:

/* Line 1455 of yacc.c  */
#line 1107 "compiler.y"
    {
					/* set vol for 2 channels */
					add_ss_channel_event(0, (yyvsp[(2) - (3)].d));
					add_ss_channel_event(1, (yyvsp[(3) - (3)].d));
				}
    break;

  case 85:

/* Line 1455 of yacc.c  */
#line 1112 "compiler.y"
    {
					/* set vol for 3 channels */
					add_ss_channel_event(0, (yyvsp[(2) - (4)].d));
					add_ss_channel_event(1, (yyvsp[(3) - (4)].d));
					add_ss_channel_event(2, (yyvsp[(4) - (4)].d));
				}
    break;

  case 86:

/* Line 1455 of yacc.c  */
#line 1118 "compiler.y"
    {
					/* set vol for 4 channels */
					add_ss_channel_event(0, (yyvsp[(2) - (5)].d));
					add_ss_channel_event(1, (yyvsp[(3) - (5)].d));
					add_ss_channel_event(2, (yyvsp[(4) - (5)].d));
					add_ss_channel_event(3, (yyvsp[(5) - (5)].d));
				}
    break;

  case 87:

/* Line 1455 of yacc.c  */
#line 1125 "compiler.y"
    {
					/* set vol for 6 channels */
					add_ss_channel_event(0, (yyvsp[(2) - (7)].d));
					add_ss_channel_event(1, (yyvsp[(3) - (7)].d));
					add_ss_channel_event(2, (yyvsp[(4) - (7)].d));
					add_ss_channel_event(3, (yyvsp[(5) - (7)].d));
					add_ss_channel_event(4, (yyvsp[(6) - (7)].d));
					add_ss_channel_event(5, (yyvsp[(7) - (7)].d));
				}
    break;

  case 88:

/* Line 1455 of yacc.c  */
#line 1134 "compiler.y"
    {
					/* delay effect */
					add_ss_eff_event(SONG_EV_SS_EFF_DELAY,
					(yyvsp[(2) - (3)].i), (yyvsp[(3) - (3)].d), 0, 0, 0, 0, 0, 0);
				}
    break;

  case 89:

/* Line 1455 of yacc.c  */
#line 1140 "compiler.y"
    {
					/* echo effect */
					add_ss_eff_event(SONG_EV_SS_EFF_ECHO,
					(yyvsp[(2) - (4)].i), (yyvsp[(3) - (4)].d), (yyvsp[(4) - (4)].d), 0, 0, 0, 0, 0);
				}
    break;

  case 90:

/* Line 1455 of yacc.c  */
#line 1146 "compiler.y"
    {
					/* comb effect */
					add_ss_eff_event(SONG_EV_SS_EFF_COMB,
					(yyvsp[(2) - (4)].i), (yyvsp[(3) - (4)].d), (yyvsp[(4) - (4)].d), 0, 0, 0, 0, 0);
				}
    break;

  case 91:

/* Line 1455 of yacc.c  */
#line 1152 "compiler.y"
    {
					/* allpass effect */
					add_ss_eff_event(SONG_EV_SS_EFF_ALLPASS,
					(yyvsp[(2) - (4)].i), (yyvsp[(3) - (4)].d), (yyvsp[(4) - (4)].d), 0, 0, 0, 0, 0);
				}
    break;

  case 92:

/* Line 1455 of yacc.c  */
#line 1158 "compiler.y"
    {
					/* flanger effect */
					add_ss_eff_event(SONG_EV_SS_EFF_FLANGER,
					(yyvsp[(2) - (7)].i), (yyvsp[(3) - (7)].d), (yyvsp[(4) - (7)].d), (yyvsp[(5) - (7)].d), (yyvsp[(6) - (7)].d), (yyvsp[(7) - (7)].d), 0, 0);
				}
    break;

  case 93:

/* Line 1455 of yacc.c  */
#line 1164 "compiler.y"
    {
					/* wobble effect */
					add_ss_eff_event(SONG_EV_SS_EFF_WOBBLE,
					(yyvsp[(2) - (4)].i), 0, 0.8, 0, (yyvsp[(3) - (4)].d), (yyvsp[(4) - (4)].d), 0, 0);
				}
    break;

  case 94:

/* Line 1455 of yacc.c  */
#line 1170 "compiler.y"
    {
					/* wobble effect, with gain */
					add_ss_eff_event(SONG_EV_SS_EFF_WOBBLE,
					(yyvsp[(2) - (5)].i), 0, (yyvsp[(5) - (5)].d), 0, (yyvsp[(3) - (5)].d), (yyvsp[(4) - (5)].d), 0, 0);
				}
    break;

  case 95:

/* Line 1455 of yacc.c  */
#line 1176 "compiler.y"
    {
					/* square wobble effect */
					add_ss_eff_event(SONG_EV_SS_EFF_SQWOBBLE,
					(yyvsp[(2) - (4)].i), 0, 0, 0, (yyvsp[(3) - (4)].d), (yyvsp[(4) - (4)].d), 0, 0);
				}
    break;

  case 96:

/* Line 1455 of yacc.c  */
#line 1182 "compiler.y"
    {
					/* half wobble effect */
					add_ss_eff_event(SONG_EV_SS_EFF_HFWOBBLE,
					(yyvsp[(2) - (4)].i), 0, 0, 0, (yyvsp[(3) - (4)].d), (yyvsp[(4) - (4)].d), 0, 0);
				}
    break;

  case 97:

/* Line 1455 of yacc.c  */
#line 1188 "compiler.y"
    {
					/* fader effect */
					add_ss_eff_event(SONG_EV_SS_EFF_FADER,
					(yyvsp[(2) - (5)].i), (yyvsp[(3) - (5)].d), 0, 0, 0, 0, (yyvsp[(4) - (5)].d), (yyvsp[(5) - (5)].d));
				}
    break;

  case 98:

/* Line 1455 of yacc.c  */
#line 1194 "compiler.y"
    {
					/* reverb effect */
					add_ss_eff_event(SONG_EV_SS_EFF_REVERB,
					(yyvsp[(2) - (2)].i), 0, 0, 0, 0, 0, 0, 0);
				}
    break;

  case 99:

/* Line 1455 of yacc.c  */
#line 1200 "compiler.y"
    {
					/* foldback distortion effect */
					add_ss_eff_event(SONG_EV_SS_EFF_FOLDBACK,
					(yyvsp[(2) - (3)].i), 0, (yyvsp[(3) - (3)].d), 0, 0, 0, 0, 0);
				}
    break;

  case 100:

/* Line 1455 of yacc.c  */
#line 1205 "compiler.y"
    {
					/* atan distortion effect */
					add_ss_eff_event(SONG_EV_SS_EFF_ATAN,
					(yyvsp[(2) - (3)].i), 0, (yyvsp[(3) - (3)].d), 0, 0, 0, 0, 0);
				}
    break;

  case 101:

/* Line 1455 of yacc.c  */
#line 1210 "compiler.y"
    {
					/* distort distortion effect */
					add_ss_eff_event(SONG_EV_SS_EFF_DISTORT,
					(yyvsp[(2) - (3)].i), 0, (yyvsp[(3) - (3)].d), 0, 0, 0, 0, 0);
				}
    break;

  case 102:

/* Line 1455 of yacc.c  */
#line 1215 "compiler.y"
    {
					/* overdrive distortion effect */
					add_ss_eff_event(SONG_EV_SS_EFF_OVERDRIVE,
					(yyvsp[(2) - (3)].i), 0, (yyvsp[(3) - (3)].d), 0, 0, 0, 0, 0);
				}
    break;

  case 103:

/* Line 1455 of yacc.c  */
#line 1221 "compiler.y"
    {
					/* off effect */
					add_ss_eff_event(SONG_EV_SS_EFF_OFF,
					(yyvsp[(2) - (2)].i), 0, 0, 0, 0, 0, 0, 0);
				}
    break;

  case 104:

/* Line 1455 of yacc.c  */
#line 1227 "compiler.y"
    {
					/* pitch stretch note */
					add_ss_pitch_stretch((yyvsp[(2) - (4)].i), (yyvsp[(3) - (4)].d), (yyvsp[(4) - (4)].d));

					forward((yyvsp[(3) - (4)].d));
				}
    break;

  case 105:

/* Line 1455 of yacc.c  */
#line 1234 "compiler.y"
    {
					/* print tempo from wave */
					add_ss_print_wave_tempo((yyvsp[(2) - (3)].i), (yyvsp[(3) - (3)].d));
				}
    break;

  case 106:

/* Line 1455 of yacc.c  */
#line 1239 "compiler.y"
    {
					/* set master volume */
					add_ss_master_volume((yyvsp[(2) - (2)].d));
				}
    break;

  case 107:

/* Line 1455 of yacc.c  */
#line 1244 "compiler.y"
    {
					/* add song info */
					add_song_info_event((yyvsp[(2) - (3)].p), (yyvsp[(3) - (3)].p));
				}
    break;

  case 108:

/* Line 1455 of yacc.c  */
#line 1249 "compiler.y"
    {
					/* midi channel */
					add_midi_channel_event((yyvsp[(2) - (2)].i));
				}
    break;

  case 109:

/* Line 1455 of yacc.c  */
#line 1254 "compiler.y"
    {
					/* midi program */
					add_midi_program_event((yyvsp[(2) - (2)].i));
				}
    break;



/* Line 1455 of yacc.c  */
#line 3409 "y.tab.c"
      default: break;
    }
  YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);

  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);

  *++yyvsp = yyval;

  /* Now `shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTOKENS] + *yyssp;
  if (0 <= yystate && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTOKENS];

  goto yynewstate;


/*------------------------------------.
| yyerrlab -- here on detecting error |
`------------------------------------*/
yyerrlab:
  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;
#if ! YYERROR_VERBOSE
      yyerror (YY_("syntax error"));
#else
      {
	YYSIZE_T yysize = yysyntax_error (0, yystate, yychar);
	if (yymsg_alloc < yysize && yymsg_alloc < YYSTACK_ALLOC_MAXIMUM)
	  {
	    YYSIZE_T yyalloc = 2 * yysize;
	    if (! (yysize <= yyalloc && yyalloc <= YYSTACK_ALLOC_MAXIMUM))
	      yyalloc = YYSTACK_ALLOC_MAXIMUM;
	    if (yymsg != yymsgbuf)
	      YYSTACK_FREE (yymsg);
	    yymsg = (char *) YYSTACK_ALLOC (yyalloc);
	    if (yymsg)
	      yymsg_alloc = yyalloc;
	    else
	      {
		yymsg = yymsgbuf;
		yymsg_alloc = sizeof yymsgbuf;
	      }
	  }

	if (0 < yysize && yysize <= yymsg_alloc)
	  {
	    (void) yysyntax_error (yymsg, yystate, yychar);
	    yyerror (yymsg);
	  }
	else
	  {
	    yyerror (YY_("syntax error"));
	    if (yysize != 0)
	      goto yyexhaustedlab;
	  }
      }
#endif
    }



  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse lookahead token after an
	 error, discard it.  */

      if (yychar <= YYEOF)
	{
	  /* Return failure if at end of input.  */
	  if (yychar == YYEOF)
	    YYABORT;
	}
      else
	{
	  yydestruct ("Error: discarding",
		      yytoken, &yylval);
	  yychar = YYEMPTY;
	}
    }

  /* Else will try to reuse lookahead token after shifting the error
     token.  */
  goto yyerrlab1;


/*---------------------------------------------------.
| yyerrorlab -- error raised explicitly by YYERROR.  |
`---------------------------------------------------*/
yyerrorlab:

  /* Pacify compilers like GCC when the user code never invokes
     YYERROR and the label yyerrorlab therefore never appears in user
     code.  */
  if (/*CONSTCOND*/ 0)
     goto yyerrorlab;

  /* Do not reclaim the symbols of the rule which action triggered
     this YYERROR.  */
  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);
  yystate = *yyssp;
  goto yyerrlab1;


/*-------------------------------------------------------------.
| yyerrlab1 -- common code for both syntax error and YYERROR.  |
`-------------------------------------------------------------*/
yyerrlab1:
  yyerrstatus = 3;	/* Each real token shifted decrements this.  */

  for (;;)
    {
      yyn = yypact[yystate];
      if (yyn != YYPACT_NINF)
	{
	  yyn += YYTERROR;
	  if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYTERROR)
	    {
	      yyn = yytable[yyn];
	      if (0 < yyn)
		break;
	    }
	}

      /* Pop the current state because it cannot handle the error token.  */
      if (yyssp == yyss)
	YYABORT;


      yydestruct ("Error: popping",
		  yystos[yystate], yyvsp);
      YYPOPSTACK (1);
      yystate = *yyssp;
      YY_STACK_PRINT (yyss, yyssp);
    }

  *++yyvsp = yylval;


  /* Shift the error token.  */
  YY_SYMBOL_PRINT ("Shifting", yystos[yyn], yyvsp, yylsp);

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturn;

/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturn;

#if !defined(yyoverflow) || YYERROR_VERBOSE
/*-------------------------------------------------.
| yyexhaustedlab -- memory exhaustion comes here.  |
`-------------------------------------------------*/
yyexhaustedlab:
  yyerror (YY_("memory exhausted"));
  yyresult = 2;
  /* Fall through.  */
#endif

yyreturn:
  if (yychar != YYEMPTY)
     yydestruct ("Cleanup: discarding lookahead",
		 yytoken, &yylval);
  /* Do not reclaim the symbols of the rule which action triggered
     this YYABORT or YYACCEPT.  */
  YYPOPSTACK (yylen);
  YY_STACK_PRINT (yyss, yyssp);
  while (yyssp != yyss)
    {
      yydestruct ("Cleanup: popping",
		  yystos[*yyssp], yyvsp);
      YYPOPSTACK (1);
    }
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif
#if YYERROR_VERBOSE
  if (yymsg != yymsgbuf)
    YYSTACK_FREE (yymsg);
#endif
  /* Make sure YYID is used.  */
  return YYID (yyresult);
}



/* Line 1675 of yacc.c  */
#line 1260 "compiler.y"


void yyerror(const char *s)
{
    c_err(s, NULL, NULL);
}


#define set_block_d(n,b) set_block(n,strdup(b))

static void compile_startup(void)
{
    track = 0;
    yyline = 1;
    compiler_error = 0;

    /* default settings */
    add_tempo_event(-2, 120.0);
    add_meter_event(-2, 4, 4);
    init_track();

    /* standard tonalities */
    set_block_d("CM", "A");
    set_block_d("Am", "$CM");
    set_block_d("C#M", "A#######");
    set_block_d("A#m", "$C#M");
    set_block_d("DM", "A#--#---");
    set_block_d("Bm", "$DM");
    set_block_d("E&M", "A--&--&&");
    set_block_d("Cm", "$E&M");
    set_block_d("EM", "A##-##--");
    set_block_d("C#m", "$EM");
    set_block_d("FM", "A------&");
    set_block_d("Dm", "$FM");
    set_block_d("F#M", "A######-");
    set_block_d("D#m", "$F#M");
    set_block_d("GM", "A---#---");
    set_block_d("Em", "$GM");
    set_block_d("A&M", "A-&&--&&");
    set_block_d("Fm", "$A&M");
    set_block_d("AM", "A#--##--");
    set_block_d("F#m", "$AM");
    set_block_d("B&M", "A--&---&");
    set_block_d("Gm", "$B&M");
    set_block_d("BM", "A##-###-");
    set_block_d("G#m", "$BM");
}


static int do_parse(void)
{
    int r = yyparse() + compiler_error;

    if (solo_track != -1)
        mute_tracks(-solo_track);

    return r;
}


int compile_ahs_string(const char *code)
{
    compile_startup();

    push_code(strdup(code));

    return do_parse();
}


int compile_ahs(const char *file)
{
    compile_startup();

    if (insert_file(file))
        return 1;

    return do_parse();
}

